mount_cleanup_basemount() {
	trap - EXIT INT QUIT TERM
	mountpoint -q "$basemount" && umount -R "$basemount"
	if [ -d "$basemount" ]; then
		rmdir "$basemount"
	fi
}

btrfs_subvol_create() {
	local new="$1"

	[ -e "$basemount/$new" ] && return
	btrfs subvolume create "$basemount/$new" >/dev/null \
		|| error "Could not create $new"
}

btrfs_subvol_delete() {
	local vol="$1"

	[ -e "$basemount/$vol" ] || return 1
	btrfs subvolume delete "$basemount/$vol" >/dev/null \
		|| error "Could not delete $vol"
}

dir_umount() {
	local dir="$1"

	mountpoint -q "$dir" || return 0
	umount "$dir" || error "Could not umount $dir"
}

ctrl_container_clear() {
	local appdev
	get_appdev

	echo "This command will remove all containers and related data."
	echo " - The following file and directories will be removed:"
	echo "   - /var/app/rollback/volumes/"
	echo "   - /var/app/volumes/"
	echo "   - /etc/atmark/containers/*.conf"
	echo " - Lines containing the word \"container\" will be deleted from the following files:"
	echo "   - /etc/sw-versions"
	echo "   - /etc/swupdate.watch"
	echo "Continue? [y/N]"
	prompt_yesno n || error "Canceled."

	sed -i -e '/container/d' /etc/swupdate.watch \
		&& persist_file /etc/swupdate.watch \
		|| error "Could not modify /etc/swupdate.watch"
	sed -i -e '/container/d' /etc/sw-versions \
		&& persist_file /etc/sw-versions \
		|| error "Could not modify /etc/sw-versions"
	rm -f /etc/atmark/containers/*.conf \
		&& persist_file -r /etc/atmark/containers \
		|| error "Could not remove container configurations /etc/atmark/containers/*.conf"

	podman_check_running
	podman pod rm -a -f >/dev/null \
		|| error "Could not remove pod"
	podman rm -a -f >/dev/null \
		|| error "Cloud not remove container"

	trap mount_cleanup_basemount EXIT INT QUIT TERM

	basemount=$(mktemp -d btrfs-root.XXXXXX) \
		|| error "Could not create tmpdir"
	mount -t btrfs "$appdev" "$basemount" \
		|| error "Could not mount appfs"

	btrfs_subvol_delete "boot_0/containers_storage"
	btrfs_subvol_delete "boot_0/volumes"
	btrfs_subvol_delete "boot_1/containers_storage"
	btrfs_subvol_delete "boot_1/volumes"
	rm -rf "$basemount"/volumes/*
	btrfs_subvol_delete "/volumes"
	if btrfs_subvol_delete "containers_storage"; then
		btrfs_subvol_create "containers_storage"
	fi

	dir_umount /var/lib/containers/storage_readonly/overlay
	dir_umount /var/lib/containers/storage_readonly
	dir_umount /var/lib/containers/storage/overlay
	dir_umount /var/lib/containers/storage
	dir_umount /var/app/rollback/volumes
	dir_umount /var/app/volumes

	btrfs_subvol_create "boot_0/containers_storage"
	btrfs_subvol_create "boot_0/volumes"
	btrfs_subvol_create "boot_1/containers_storage"
	btrfs_subvol_create "boot_1/volumes"
	btrfs_subvol_create "volumes"

	mount /var/lib/containers/storage_readonly \
		|| error "Could not mount containers_storage subvol"
	mount /var/app/rollback/volumes \
		|| error "Could not mount rollback/volume subvol"
	mount /var/app/volumes \
		|| error "Could not mount volume subvol"
	if [ -e "$basemount/containers_storage" ]; then
		mount /var/lib/containers/storage \
			|| error "Could not mount persistent storage"
	fi

	umount "$basemount" \
		|| error "Could not umount tmpdir $basemount"
	rmdir "$basemount" \
		|| error "Could not remove tmpdir $basemount"
	echo "Remove all container data succeeded"
}
